/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WBLOWING_H
#define WBLOWING_H

#include "dart.h"
#include "wGroup.h"
#include "wF0El.h"
#include <vector>

namespace dart
{

/**
 * @brief Manage blowing boundary condition
 * @authors Adrien Crovato
 */
class DART_API Blowing : public tbox::Group
{
public:
    std::vector<tbox::Node *> nodes; ///< nodes of Blowing
#ifndef SWIG
    std::vector<std::pair<tbox::Element *, F0ElC *>> uE; ///< blowing velocity for each element
#endif

    Blowing(std::shared_ptr<tbox::MshData> _msh, int nos);
    Blowing(std::shared_ptr<tbox::MshData> _msh, std::string const &names);
    virtual ~Blowing();

    void setU(size_t i, double ue);

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void create();
};

} // namespace dart

#endif //WBLOWING_H
