/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WBOUNDARY_H
#define WBOUNDARY_H

#include "dart.h"
#include "wGroups.h"

#include <vector>
#include <map>
#include <Eigen/Dense>

namespace dart
{

/**
 * @brief Handle the boundary of a medium
 * @authors Adrien Crovato
 */
class DART_API Boundary : public tbox::Groups
{
public:
    std::vector<tbox::Node *> nodes;                  ///< nodes of Boundary
    std::map<tbox::Node *, size_t> nMap;              ///< node to local index map
    std::map<tbox::Element *, tbox::Element *> svMap; ///< surface to volume element map
    double Cl;                                        ///< lift coefficient
    double Cd;                                        ///< drag coefficient
    double Cs;                                        ///< sideforce coefficient
    double Cm;                                        ///< pitch moment coefficient (positive nose-up)
    std::vector<Eigen::Vector3d> nLoads;              ///< nodal (normalized) aerodynamic loads on boundary

    Boundary(std::shared_ptr<tbox::MshData> _msh, std::vector<int> const &nos);
    Boundary(std::shared_ptr<tbox::MshData> _msh, std::vector<std::string> const &names);
    virtual ~Boundary() { std::cout << "~Boundary()\n"; }

    void save(std::string const &name, tbox::Results const &res);

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void create();
};

} // namespace dart

#endif //WBOUNDARY_H
